/* dc_gui2 - a GTK+2 GUI for DCTC
 * Copyright (C) 2002 Eric Prevoteau
 *
 * done_clist.c: Copyright (C) Eric Prevoteau <www@ac2i.tzo.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/*
$Id: done_clist.c,v 1.2 2003/01/21 08:46:15 eric Exp $
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <db.h>
#include <gnome.h>

#include "gtkcellrenderertextx.h"
#include "misc_gtk.h"
#include "done_clist.h"
#include "main.h"
#include "gui_define.h"
#include "macro.h"
#include "unode.h"
#include "init_fnc.h"
#include "dctc_process.h"

/********************************************/
/* load the .done file list into done_clist */
/********************************************/
void load_done_xfer(char *fname,int with_clear)
{
	GtkWidget *w;
	FILE *f;
	char buf[51200];
	GtkTreeModel *gtm;
	GtkListStore *gls;
	GtkTreeIter iter;

	w=get_widget_by_widget_name(main_window,"done_clist");

	if(w==NULL)
		return;

	gls=GTK_LIST_STORE(gtm=gtk_tree_view_get_model(GTK_TREE_VIEW(w)));

	if(with_clear==1)
	{
		gtk_list_store_clear(gls);
	}

	f=fopen(fname,"rb");
	if(f==NULL)
		return;

	while(fgets(buf,sizeof(buf),f)!=NULL)
	{
		char *t;

		t=strrchr(buf,'\n');
		if(t!=NULL)
			*t='\0';

		/* string format is: nick|DL/remote_fic$v|local_fic|start_pos|filesize|start_time */
		/*               or: nick|XDL|local_fic|filesize */

		t=strstr(buf,"|DL/");
		if(t==NULL)
		{
			/* not a |DL/, maybe a |XDL| */
			t=strstr(buf,"|XDL|");
			if(t!=NULL)
			{
				gchar **fields;

				fields=g_strsplit(buf,"|",0);
				if(gchar_array_len(fields)==4)
				{
					if(get_iter_from_string(gtm,DOC_LFNAME_COL,fields[2],&iter)==FALSE)
					{
						gulong size_as_val=strtoul(fields[3],NULL,10);
						char buf_size[64];
#ifndef NO_PRINTF_LOCALE
						sprintf(buf_size,"%'lu",size_as_val);  /* NO_PRINTF_LOCAL support added */
#else
						sprintf(buf_size,"%lu",size_as_val);
#endif

						gtk_list_store_append(gls,&iter);
						if(utf8_mode==TRUE)
						{
							gtk_list_store_set(gls,&iter,DOC_NICK_COL,fields[0],		/* nickname */
																	DOC_SIZE_COL,buf_size,
																	DOC_SIZE_AS_VAL,size_as_val,		/* local filesize */
																	DOC_FNAME_COL,_("XDL (multiple sources)"),			/* remote filename */
																	DOC_LFNAME_COL,fields[2],		/* local filename */
																	DOC_NICK,fields[0],
																	-1);
						}
						else
						{
							gchar *utf8_0, *utf8_2;

							utf8_0=g_locale_to_utf8(fields[0],-1,NULL,NULL,NULL);
							utf8_2=g_locale_to_utf8(fields[2],-1,NULL,NULL,NULL);
							gtk_list_store_set(gls,&iter,DOC_NICK_COL,utf8_0,		/* nickname */
																	DOC_SIZE_COL,buf_size,
																	DOC_SIZE_AS_VAL,size_as_val,		/* local filesize */
																	DOC_FNAME_COL,_("XDL (multiple sources)"),			/* remote filename */
																	DOC_LFNAME_COL,utf8_2,		/* local filename */
																	DOC_NICK,fields[0],
																	-1);

							g_free(utf8_0);
							g_free(utf8_2);
						}
					}
				}
				g_strfreev(fields);
			}
		}
	}
	fclose(f);
}

/********************************************/
/* build modele and view for the done clist */
/********************************************/
void bmav4_done_clist(void)
{
	GtkListStore *model;
	GtkWidget *view;
	GtkCellRenderer *rend;
	GtkTreeSelection *slc;

	model=gtk_list_store_new(NB_DOC_COL,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING,
													G_TYPE_STRING,G_TYPE_ULONG, G_TYPE_STRING);

	view=get_widget_by_widget_name(main_window,"done_clist");

	gtk_tree_view_set_model(GTK_TREE_VIEW(view),GTK_TREE_MODEL(model));
	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(view),TRUE);
#if 0
	gtk_tree_view_set_headers_clickable(GTK_TREE_VIEW(view),TRUE);
#endif

	slc=gtk_tree_view_get_selection(GTK_TREE_VIEW(view));
	gtk_tree_selection_set_mode(slc,GTK_SELECTION_MULTIPLE);
	
	/* the first column of the view display the first column of the model, and so on. The 4th entry of the model is not displayed */
	rend=gtk_cell_renderer_textx_new();
	g_object_set(rend,"rowspacing",TRUE,NULL);
	MY_RS_gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),DOC_NICK_COL,_("Nick"),rend,"text",DOC_NICK_COL,NULL);

	rend=gtk_cell_renderer_textx_new();
	g_object_set(rend,"rowspacing",TRUE,NULL);
	g_object_set(rend,"xalign",(gfloat)1.0,NULL);
	MY_RS2_gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),DOC_SIZE_COL,DOC_SIZE_AS_VAL,_("Size"),rend,"text",DOC_SIZE_COL,NULL);

	rend=gtk_cell_renderer_textx_new();
	g_object_set(rend,"rowspacing",TRUE,NULL);
	MY_RS_gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),DOC_FNAME_COL,_("Filename"),rend,"text",DOC_FNAME_COL,NULL);

	rend=gtk_cell_renderer_textx_new();
	g_object_set(rend,"rowspacing",TRUE,NULL);
	MY_RS_gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(view),DOC_LFNAME_COL,_("Local Filename"),rend,"text",DOC_LFNAME_COL,NULL);

	/* The view now holds a reference.  We can get rid of our own
	 * reference */
	g_object_unref (G_OBJECT (model));
}

